# import sys

# from PyQt6.QtWidgets import QApplication

# from starrail.gui.application import StarRailToolkitWindow


# def gui_entry():
#     app = QApplication(sys.argv)
#     _ = StarRailToolkitWindow()
#     sys.exit(app.exec())


# if __name__ == '__main__':
#     gui_entry()



import json

import pandas as pd

import starrail.utils.babelfish as babelfish
from starrail.gacha.database import DatabaseFactory
from starrail.gacha.parse import GachaDataManager
from starrail.gacha.type import GachaType
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


def export_as_sql(manager: GachaDataManager, output_path: str) -> None:
    logger.info(f'Exporting data to cache {output_path}')
    with DatabaseFactory.get_database(output_path) as db:
        for gacha_type in GachaType:
            should_insert = []
            for entry in manager.gacha[gacha_type.value]:
                if 'existing' in entry and not entry['existing']:
                    should_insert.append(entry)
            logger.info(
                f'Exporting {gacha_type.name}, '
                f'totally {len(should_insert)} new items',
            )
            rev_insert = reversed(should_insert)
            for entry in rev_insert:
                db.add_entry(gacha_type.name, entry)


def export_as_json(manager: GachaDataManager, output_path: str) -> None:
    data_dict = dict()
    for gacha_type in GachaType:
        data_dict[gacha_type.name] = manager.gacha[gacha_type.value].tolist()
    with open(output_path, 'w', encoding='utf-8') as fout:
        json.dump(data_dict, fout, indent=2, ensure_ascii=False)


def export_as_xlsx(manager: GachaDataManager, output_path: str) -> None:
    with pd.ExcelWriter(output_path) as writer:
        for gacha_type in GachaType:
            df = pd.json_normalize(manager.gacha[gacha_type.value].tolist())
            df.to_excel(writer, sheet_name=gacha_type.name)


def export_as_csv(manager: GachaDataManager, output_path: str) -> None:
    data_list = []
    for gacha_type in GachaType:
        data_list.extend(manager.gacha[gacha_type.value].tolist())
    df = pd.json_normalize(data_list)
    df.to_csv(output_path, encoding='utf-8')


def export_as_md(manager: GachaDataManager, output_path: str) -> None:
    md = f'# {babelfish.gacha_report()}\n\n'
    for gacha_type in GachaType:
        md += f'## {babelfish.translate(gacha_type.name)}\n\n'
        md += babelfish.markdown_thead()
        stats = manager.gacha[gacha_type.value].stats
        for item in stats:
            rtype = item['rank_type']
            count = item['count']
            basic = item['basic_prob']
            compr = item['compr_prob']
            since_last = item['since_last']
            md += f'|{rtype}|{count}|{basic}|{compr}|{since_last}|\n'
        md += '\n'
        if stats[0]['attempts']:
            attempt_string = ' '.join(stats[0]['attempts'])
            average = stats[0]['average']
            md += f'{babelfish.history_of_5_stars()}: '
            md += f'**{attempt_string}**\n\n'
            md += f'{babelfish.average_gacha_per_5_star()}: **{average}**\n\n'
    with open(output_path, 'w', encoding='utf-8') as fout:
        fout.write(md)


def export_as_html(manager: GachaDataManager, output_path: str) -> None:
    title = f'{babelfish.gacha_title(manager.uid)}'
    content = f'<h1>{babelfish.gacha_report()}</h1>\n'
    for gacha_type in GachaType:
        content += f'<h2>{babelfish.translate(gacha_type.name)}</h2>\n'
        thead = babelfish.html_thead()
        table = f'<table><thead><tr>{thead}</tr></thead><tbody>\n'
        stats = manager.gacha[gacha_type.value].stats
        for item in stats:
            rtype = item['rank_type']
            count = item['count']
            basic = item['basic_prob']
            compr = item['compr_prob']
            since_last = item['since_last']
            table += (
                f'<tr><td>{rtype}</td><td>{count}</td><td>{basic}</td>'
                f'<td>{compr}</td><td>{since_last}</td></tr>\n'
            )
        table += '</tbody></table>\n'
        content += table
        if stats[0]['attempts']:
            attempt_string = ' '.join(stats[0]['attempts'])
            average = stats[0]['average']
            content += f'<p>{babelfish.history_of_5_stars()}: '
            content += f'<b>{attempt_string}</b></p>\n'
            content += f'<p>{babelfish.average_gacha_per_5_star()}: '
            content += f'<b>{average}</b></p>\n'
    style = (
        '<style type="text/css">'
        r'html {font-family: sans-serif;} '
        r'body {margin: 10px;} '
        r'table {border-collapse: collapse; border-spacing: 0;'
        r'       empty-cells: show; border: 1px solid #cbcbcb;} '
        r'td, th {padding: 0; border-left: 1px solid #cbcbcb;'
        r'        border-width: 0 0 0 1px; font-size: inherit;'
        r'        margin: 0; overflow: visible; padding: .5em 1em;} '
        r'thead {background-color: #e0e0e0; color: #000;'
        r'       text-align: left; vertical-align: bottom; }'
        r'tr:nth-child(odd) td {background-color: transparent;} '
        r'tr:nth-child(even) td {background-color: #f2f2f2;} '
        '</style>'
    )
    html = (
        f'<html lang="zh"><head><title>{title}</title><meta charset="UTF-8">'
        f'{style}</head><body>{content}</body></html>'
    )
    with open(output_path, 'w', encoding='utf-8') as fout:
        fout.write(html)



import copy
import json
import os

from easydict import EasyDict

userroot = os.path.abspath(os.path.expanduser('~'))

configuration = EasyDict(
    cache_dir=os.path.join(userroot, '.starrail'),
    config_path=os.path.join(userroot, '.starrail', 'config.json'),
    check_update=True,
)


def export_config(cfg, skip_keys=[]):
    config_path = cfg.config_path
    cfg = copy.deepcopy(cfg)
    for key in skip_keys:
        if key in cfg:
            cfg.pop(key)
    with open(config_path, 'w', encoding='utf-8') as fcfg:
        json.dump(cfg, fcfg, indent=2, ensure_ascii=False)


def init_config():

    os.makedirs(configuration.cache_dir, exist_ok=True)

    if os.path.isfile(configuration.config_path):
        with open(configuration.config_path, encoding='utf-8') as fcfg:
            custom_config = json.load(fcfg)

        configuration.update(custom_config)
    else:
        export_config(
            configuration,
            skip_keys=['cache_dir', 'config_path'],
        )



import os
from datetime import datetime

import starrail.gacha.fileio as fileio
from starrail.gacha.fetch import fetch_json
from starrail.gacha.parse import GachaDataManager
from starrail.gacha.type import GachaType
from starrail.gacha.url import get_api_url, get_url_template
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


def integers():
    r = 1
    while True:
        yield r
        r += 1


def check_response(payload, code):
    if payload is None or not 200 <= code < 300:
        logger.info(f'Whether payload is None: {payload is None}')
        logger.info(f'Status code: {code}')
        return False
    if 'data' not in payload or 'list' not in payload['data']:
        logger.info(f'Whether payload contains `data`: {"data" in payload}')
        logger.info(
            f'Whether payload data contains `list`:'
            f'{"list" in payload["data"]}',
        )
        return False
    if not payload['data']['list']:
        logger.info(
            f'Length of payload.data.list: '
            f'{len(payload["data"]["list"])}',
        )
        return False
    return True


def export_gacha_type(api_template: str, gacha_type: GachaType):
    r = []
    end_id = '0'
    for page in integers():
        logger.info(f'Downloading page {page} of type {gacha_type.name}')
        api_url = get_api_url(
            api_template, end_id, str(gacha_type.value),
            str(page), '5',
        )
        logger.debug(f'Requesting {api_url}')
        response, code = fetch_json(api_url)
        if not check_response(response, code):
            break
        data_list = response['data']['list']
        r.extend(data_list)
        end_id = data_list[-1]['id']
    return r


def export_gacha_from_api(api_url, export):
    assert api_url, 'API URL should be provided, auto-detect is coming soon'
    response, code = fetch_json(api_url)
    valid = check_response(response, code)
    if not valid:
        logger.fatal('Error while checking response from api URL, exitting')
        raise ValueError('Invalid api URL, please check your input')

    api_template = get_url_template(api_url)

    uid = response['data']['list'][0]['uid']
    manager = GachaDataManager(uid)
    logger.info(f'Successfully connected to cache of uid {uid}')
    manager.log_stats()

    for gacha_type in GachaType:
        records = export_gacha_type(api_template, gacha_type)
        manager.add_records(gacha_type.value, records)
        manager.gacha[gacha_type.value].sort()
        logger.info(f'Finish downloading records of {gacha_type.name}')

    fileio.export_as_sql(manager, manager.cache_path)

    manager.log_stats()

    export_hooks = dict(
        csv=fileio.export_as_csv,
        html=fileio.export_as_html,
        json=fileio.export_as_json,
        md=fileio.export_as_md,
        xlsx=fileio.export_as_xlsx,
    )
    if 'all' in export:
        export = ['csv', 'html', 'json', 'md', 'xlsx']

    timestamp = datetime.now().strftime('%Y%m%d%H%M%S')

    for format in export:
        logger.info(f'Exporting gacha data as {format} format')
        output_dir = os.getcwd()
        filename = f'HKSR-export-{uid}-{timestamp}.{format}'
        path = os.path.join(output_dir, filename)
        export_hooks[format](manager, path)
        logger.info(f'Gacha data in {format} format is saved to {path}')



import starrail.utils.babelfish.dictionary as dictionary
from starrail.utils.babelfish.translate import translate

average_gacha_per_5_star = dictionary.average_gacha_per_5_star
gacha_report = dictionary.gacha_report
gacha_title = dictionary.gacha_title
history_of_5_stars = dictionary.history_of_5_stars
html_thead = dictionary.html_thead
markdown_thead = dictionary.markdown_thead

__all__ = ['translate']



import argparse

from starrail.config import configuration as cfg
from starrail.entry.setup import setup
from starrail.gacha.service import export_gacha_from_api
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


def parse_args():
    parser = argparse.ArgumentParser(
        prog='hksr',
        description='Honkai: Star Rail Toolkit',
    )
    parser.add_argument(
        '--api', type=str,
        help='URL of the gacha api, please refer to README.md for details.',
    )
    parser.add_argument(
        '--export', nargs='+', type=str,
        choices=['all', 'csv', 'html', 'json', 'md', 'xlsx'],
        help='Types of expected export formats.',
    )
    parser.add_argument(
        '--locale', type=str, default='zhs',
        choices=['en', 'zhs'],
        help=(
            'Language of gacha report. Abbreviations: `en` for English, '
            '`zhs` for simplified Chinese'
        ),
    )
    parser.add_argument(
        '--log-level', type=str, default='DEBUG',
        choices=['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'],
        help='Controlling the level of logging output.',
    )

    return parser.parse_args()


def cli_entry():
    args = parse_args()
    setup(log_level=args.log_level)
    logger.info(args)
    logger.info(cfg)
    export_gacha_from_api(api_url=args.api, export=args.export)



from starrail.utils.babelfish.multilingual import MultilingualString as _MS

# Lookup table - used by dictionary

lookup_table = dict(
    STELLAR=_MS(en='Stellar Warp', zhs='群星跃迁'),
    DEPARTURE=_MS(en='Departure Warp', zhs='始发跃迁'),
    CHARACTER=_MS(en='Character Event Warp', zhs='角色跃迁'),
    LIGHT_CONE=_MS(en='Light Cone Event Warp', zhs='光锥跃迁'),
)

# Pre-defined vocabularies

average_gacha_per_5_star = _MS(
    en='Average gacha per 5-star',
    zhs='五星平均抽数',
)

gacha_report = _MS(en='Gacha Report', zhs='抽卡分析')

gacha_title = _MS(
    en='Gacha Report for Honkai: Star Rail Player {}',
    zhs='《崩坏：星穹铁道》玩家{}的抽卡分析',
)

history_of_5_stars = _MS(
    en='History of 5-star gacha attempts',
    zhs='五星出货记录',
)

html_thead = _MS(
    en=(
        '<th>Type</th><th>Count</th><th>Basic Prob.</th><th>True Prob.</th>'
        '<th>Since Last</th>'
    ),
    zhs=('<th>星级</th><th>数量</th><th>基础概率</th><th>综合概率</th><th>距上次</th>'),
)

markdown_thead = _MS(
    en=(
        '| Type | Count | Basic Prob. | True Prob. | Since Last |\n'
        '| ---- | ----- | ----------- | ---------- | ---------- |\n'
    ),
    zhs=(
        '| 星级 | 数量 | 基础概率 | 综合概率 | 距上次 |\n'
        '| ---- | ---- | ------ | ------- | ---- |\n'
    ),
)
